@@ -550,7 +550,7 @@ class ExtensionApp(JupyterApp):
     serverapp_class = ServerApp
 
     @classmethod
-    def make_serverapp(cls, **kwargs):
+    def make_serverapp(cls, **kwargs: t.Any) -> ServerApp:
         """Instantiate the ServerApp
 
         Override to customize the ServerApp before it loads any configuration
@@ -560,7 +560,7 @@ class ExtensionApp(JupyterApp):
     @classmethod
     def initialize_server(cls, argv=None, load_other_extensions=True, **kwargs):
         """Creates an instance of ServerApp and explicitly sets
-        this extension to enabled=True (i.e. superceding disabling
+        this extension to enabled=True (i.e. superseding disabling
         found in other config from files).
 
         The `launch_instance` method uses this method to initialize
@@ -573,7 +573,7 @@ class ExtensionApp(JupyterApp):
             cls.serverapp_config["jpserver_extensions"] = jpserver_extensions
             find_extensions = False
         serverapp = cls.make_serverapp(jpserver_extensions=jpserver_extensions, **kwargs)
-        serverapp.aliases.update(cls.aliases)
+        serverapp.aliases.update(cls.aliases)  # type:ignore[has-type]
         serverapp.initialize(
             argv=argv or [],
             starter_extension=cls.name,
@@ -608,10 +608,7 @@ class ExtensionApp(JupyterApp):
 
         # Log if extension is blocking other extensions from loading.
         if not cls.load_other_extensions:
-            serverapp.log.info(
-                "{ext_name} is running without loading "
-                "other extensions.".format(ext_name=cls.name)
-            )
+            serverapp.log.info(f"{cls.name} is running without loading other extensions.")
         # Start the server.
         try:
             serverapp.start()
