@@ -11,19 +11,22 @@ from threading import Thread
 from time import monotonic
 from typing import Any, Dict, Optional
 
-import websocket  # type:ignore
+import websocket  # type:ignore[import-untyped]
 from jupyter_client.asynchronous.client import AsyncKernelClient
 from jupyter_client.clientabc import KernelClientABC
 from jupyter_client.kernelspec import KernelSpecManager
-from jupyter_client.manager import AsyncKernelManager
 from jupyter_client.managerabc import KernelManagerABC
 from jupyter_core.utils import ensure_async
 from tornado import web
 from tornado.escape import json_decode, json_encode, url_escape, utf8
 from traitlets import DottedObjectName, Instance, Type, default
 
-from .._tz import UTC
-from ..services.kernels.kernelmanager import AsyncMappingKernelManager
+from .._tz import UTC, utcnow
+from ..services.kernels.kernelmanager import (
+    AsyncMappingKernelManager,
+    ServerKernelManager,
+    emit_kernel_action_event,
+)
 from ..services.sessions.sessionmanager import SessionManager
 from ..utils import url_path_join
 from .gateway_client import GatewayClient, gateway_request
@@ -47,7 +50,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         """Initialize a gateway mapping kernel manager."""
         super().__init__(**kwargs)
         self.kernels_url = url_path_join(
-            GatewayClient.instance().url, GatewayClient.instance().kernels_endpoint
+            GatewayClient.instance().url or "", GatewayClient.instance().kernels_endpoint or ""
         )
 
     def remove_kernel(self, kernel_id):
@@ -79,7 +82,6 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         await km.start_kernel(kernel_id=kernel_id, **kwargs)
         kernel_id = km.kernel_id
         self._kernels[kernel_id] = km
-
         # Initialize culling if not already
         if not self._initialized_culler:
             self.initialize_culler()
@@ -97,7 +99,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         """
         model = None
         km = self.get_kernel(str(kernel_id))
-        if km:
+        if km:  # type:ignore[truthy-bool]
             model = km.kernel  # type:ignore[attr-defined]
         return model
 
@@ -123,11 +125,34 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         culled_ids = []
         for kid, _ in our_kernels.items():
             if kid not in kernel_models:
+                # The upstream kernel was not reported in the list of kernels.
                 self.log.warning(
-                    f"Kernel {kid} no longer active - probably culled on Gateway server."
+                    f"Kernel {kid} not present in the list of kernels - possibly culled on Gateway server."
                 )
-                self._kernels.pop(kid, None)
-                culled_ids.append(kid)  # TODO: Figure out what do with these.
+                try:
+                    # Try to directly refresh the model for this specific kernel in case
+                    # the upstream list of kernels was erroneously incomplete.
+                    #
+                    # That might happen if the case of a proxy that manages multiple
+                    # backends where there could be transient connectivity issues with
+                    # a single backend.
+                    #
+                    # Alternatively, it could happen if there is simply a bug in the
+                    # upstream gateway server.
+                    #
+                    # Either way, including this check improves our reliability in the
+                    # face of such scenarios.
+                    model = await self._kernels[kid].refresh_model()
+                except web.HTTPError:
+                    model = None
+                if model:
+                    kernel_models[kid] = model
+                else:
+                    self.log.warning(
+                        f"Kernel {kid} no longer active - probably culled on Gateway server."
+                    )
+                    self._kernels.pop(kid, None)
+                    culled_ids.append(kid)  # TODO: Figure out what do with these.
         return list(kernel_models.values())
 
     async def shutdown_kernel(self, kernel_id, now=False, restart=False):
@@ -189,12 +214,12 @@ class GatewayKernelSpecManager(KernelSpecManager):
         """Initialize a gateway kernel spec manager."""
         super().__init__(**kwargs)
         base_endpoint = url_path_join(
-            GatewayClient.instance().url, GatewayClient.instance().kernelspecs_endpoint
+            GatewayClient.instance().url or "", GatewayClient.instance().kernelspecs_endpoint
         )
 
         self.base_endpoint = GatewayKernelSpecManager._get_endpoint_for_user_filter(base_endpoint)
         self.base_resource_endpoint = url_path_join(
-            GatewayClient.instance().url,
+            GatewayClient.instance().url or "",
             GatewayClient.instance().kernelspecs_resource_endpoint,
         )
 
@@ -211,6 +236,8 @@ class GatewayKernelSpecManager(KernelSpecManager):
         This enables clients to properly route through jupyter_server to a gateway
         for kernel resources such as logo files
         """
+        if not self.parent:
+            return {}
         kernelspecs = kernel_specs["kernelspecs"]
         for kernel_name in kernelspecs:
             resources = kernelspecs[kernel_name]["resources"]
@@ -248,6 +275,8 @@ class GatewayKernelSpecManager(KernelSpecManager):
         # If different log a warning and reset the default.  However, the
         # caller of this method will still return this server's value until
         # the next fetch of kernelspecs - at which time they'll match.
+        if not self.parent:
+            return {}
         km = self.parent.kernel_manager
         remote_default_kernel_name = fetched_kspecs.get("default")
         if remote_default_kernel_name != km.default_kernel_name:
@@ -282,7 +311,7 @@ class GatewayKernelSpecManager(KernelSpecManager):
         try:
             response = await gateway_request(kernel_spec_url, method="GET")
         except web.HTTPError as error:
-            if error.status_code == 404:  # noqa[PLR2004]
+            if error.status_code == 404:  # noqa: PLR2004
                 # Convert not found to KeyError since that's what the Notebook handler expects
                 # message is not used, but might as well make it useful for troubleshooting
                 msg = f"kernelspec {kernel_name} not found on Gateway server at: {GatewayClient.instance().url}"
@@ -311,7 +340,7 @@ class GatewayKernelSpecManager(KernelSpecManager):
         try:
             response = await gateway_request(kernel_spec_resource_url, method="GET")
         except web.HTTPError as error:
-            if error.status_code == 404:  # noqa[PLR2004]
+            if error.status_code == 404:  # noqa: PLR2004
                 kernel_spec_resource = None
             else:
                 raise
@@ -343,7 +372,7 @@ class GatewaySessionManager(SessionManager):
         return km is None
 
 
-class GatewayKernelManager(AsyncKernelManager):
+class GatewayKernelManager(ServerKernelManager):
     """Manages a single kernel remotely via a Gateway Server."""
 
     kernel_id: Optional[str] = None  # type:ignore[assignment]
@@ -357,12 +386,13 @@ class GatewayKernelManager(AsyncKernelManager):
         """Initialize the gateway kernel manager."""
         super().__init__(**kwargs)
         self.kernels_url = url_path_join(
-            GatewayClient.instance().url, GatewayClient.instance().kernels_endpoint
+            GatewayClient.instance().url or "", GatewayClient.instance().kernels_endpoint
         )
         self.kernel_url: str
         self.kernel = self.kernel_id = None
         # simulate busy/activity markers:
-        self.execution_state = self.last_activity = None
+        self.execution_state = "starting"
+        self.last_activity = utcnow()
 
     @property
     def has_kernel(self):
@@ -407,7 +437,7 @@ class GatewayKernelManager(AsyncKernelManager):
                 response = await gateway_request(self.kernel_url, method="GET")
 
             except web.HTTPError as error:
-                if error.status_code == 404:  # noqa[PLR2004]
+                if error.status_code == 404:  # noqa: PLR2004
                     self.log.warning("Kernel not found at: %s" % self.kernel_url)
                     model = None
                 else:
@@ -435,6 +465,9 @@ class GatewayKernelManager(AsyncKernelManager):
     # Kernel management
     # --------------------------------------------------------------------------
 
+    @emit_kernel_action_event(
+        success_msg="Kernel {kernel_id} was started.",
+    )
     async def start_kernel(self, **kwargs):
         """Starts a kernel via HTTP in an asynchronous manner.
 
@@ -452,7 +485,7 @@ class GatewayKernelManager(AsyncKernelManager):
 
             # Let KERNEL_USERNAME take precedent over http_user config option.
             if os.environ.get("KERNEL_USERNAME") is None and GatewayClient.instance().http_user:
-                os.environ["KERNEL_USERNAME"] = GatewayClient.instance().http_user
+                os.environ["KERNEL_USERNAME"] = GatewayClient.instance().http_user or ""
 
             payload_envs = os.environ.copy()
             payload_envs.update(kwargs.get("env", {}))  # Add any env entries in this request
@@ -486,6 +519,9 @@ class GatewayKernelManager(AsyncKernelManager):
             self.kernel = await self.refresh_model()
             self.log.info(f"GatewayKernelManager using existing kernel: {self.kernel_id}")
 
+    @emit_kernel_action_event(
+        success_msg="Kernel {kernel_id} was shutdown.",
+    )
     async def shutdown_kernel(self, now=False, restart=False):
         """Attempts to stop the kernel process cleanly via HTTP."""
 
@@ -495,11 +531,14 @@ class GatewayKernelManager(AsyncKernelManager):
                 response = await gateway_request(self.kernel_url, method="DELETE")
                 self.log.debug("Shutdown kernel response: %d %s", response.code, response.reason)
             except web.HTTPError as error:
-                if error.status_code == 404:  # noqa[PLR2004]
+                if error.status_code == 404:  # noqa: PLR2004
                     self.log.debug("Shutdown kernel response: kernel not found (ignored)")
                 else:
                     raise
 
+    @emit_kernel_action_event(
+        success_msg="Kernel {kernel_id} was restarted.",
+    )
     async def restart_kernel(self, **kw):
         """Restarts a kernel via HTTP."""
         if self.has_kernel:
@@ -514,6 +553,9 @@ class GatewayKernelManager(AsyncKernelManager):
             )
             self.log.debug("Restart kernel response: %d %s", response.code, response.reason)
 
+    @emit_kernel_action_event(
+        success_msg="Kernel {kernel_id} was interrupted.",
+    )
     async def interrupt_kernel(self):
         """Interrupts the kernel via an HTTP request."""
         if self.has_kernel:
@@ -533,8 +575,10 @@ class GatewayKernelManager(AsyncKernelManager):
         if self.has_kernel:
             # Go ahead and issue a request to get the kernel
             self.kernel = await self.refresh_model()
+            self.log.debug(f"The kernel: {self.kernel} is alive.")
             return True
         else:  # we don't have a kernel
+            self.log.debug(f"The kernel: {self.kernel} no longer exists.")
             return False
 
     def cleanup_resources(self, restart=False):
@@ -638,7 +682,7 @@ class ChannelQueue(Queue):
 
 
 class HBChannelQueue(ChannelQueue):
-    """A queue for the hearbeat channel."""
+    """A queue for the heartbeat channel."""
 
     def is_beating(self) -> bool:
         """Whether the channel is beating."""
@@ -695,7 +739,7 @@ class GatewayKernelClient(AsyncKernelClient):
         """
 
         ws_url = url_path_join(
-            GatewayClient.instance().ws_url,
+            GatewayClient.instance().ws_url or "",
             GatewayClient.instance().kernels_endpoint,
             url_escape(self.kernel_id),
             "channels",
