@@ -15,6 +15,7 @@ from anyio.to_thread import run_sync
 from tornado.web import HTTPError
 from traitlets import Bool
 from traitlets.config import Configurable
+from traitlets.config.configurable import LoggingConfigurable
 
 from jupyter_server.utils import ApiPath, to_api_path, to_os_path
 
@@ -165,7 +166,7 @@ def _simple_writing(path, text=True, encoding="utf-8", log=None, **kwargs):
     fileobj.close()
 
 
-class FileManagerMixin(Configurable):
+class FileManagerMixin(LoggingConfigurable, Configurable):
     """
     Mixin for ContentsAPI classes that interact with the filesystem.
 
@@ -187,7 +188,7 @@ class FileManagerMixin(Configurable):
         True,
         config=True,
         help="""By default notebooks are saved on disk on a temporary file and then if succefully written, it replaces the old ones.
-      This procedure, namely 'atomic_writing', causes some bugs on file system whitout operation order enforcement (like some networked fs).
+      This procedure, namely 'atomic_writing', causes some bugs on file system without operation order enforcement (like some networked fs).
       If set to False, the new notebook is written directly on the old one which could fail (eg: full filesystem or quota )""",
     )
 
@@ -203,7 +204,7 @@ class FileManagerMixin(Configurable):
         Depending on flag 'use_atomic_writing', the wrapper perform an actual atomic writing or
         simply writes the file (whatever an old exists or not)"""
         with self.perm_to_403(os_path):
-            kwargs["log"] = self.log  # type:ignore
+            kwargs["log"] = self.log
             if self.use_atomic_writing:
                 with atomic_writing(os_path, *args, **kwargs) as f:
                     yield f
@@ -223,7 +224,7 @@ class FileManagerMixin(Configurable):
                 # but nobody should be doing that anyway.
                 if not os_path:
                     os_path = e.filename or "unknown file"
-                path = to_api_path(os_path, root=self.root_dir)  # type:ignore
+                path = to_api_path(os_path, root=self.root_dir)  # type:ignore[attr-defined]
                 raise HTTPError(403, "Permission denied: %s" % path) from e
             else:
                 raise
@@ -233,7 +234,7 @@ class FileManagerMixin(Configurable):
 
         like shutil.copy2, but log errors in copystat
         """
-        copy2_safe(src, dest, log=self.log)  # type:ignore
+        copy2_safe(src, dest, log=self.log)
 
     def _get_os_path(self, path):
         """Given an API path, return its file system path.
@@ -252,7 +253,8 @@ class FileManagerMixin(Configurable):
         ------
         404: if path is outside root
         """
-        root = os.path.abspath(self.root_dir)  # type:ignore
+        self.log.debug("Reading path from disk: %s", path)
+        root = os.path.abspath(self.root_dir)  # type:ignore[attr-defined]
         # to_os_path is not safe if path starts with a drive, since os.path.join discards first part
         if os.path.splitdrive(path)[0]:
             raise HTTPError(404, "%s is not a relative API path" % path)
@@ -359,7 +361,7 @@ class AsyncFileManagerMixin(FileManagerMixin):
 
         like shutil.copy2, but log errors in copystat
         """
-        await async_copy2_safe(src, dest, log=self.log)  # type:ignore
+        await async_copy2_safe(src, dest, log=self.log)
 
     async def _read_notebook(self, os_path, as_version=4, capture_validation_error=None):
         """Read a notebook from an os path."""
